class UriUtils {

  /// 给 url 添加一个 scheme ———— 如果它没有的话
  static Uri fixUriScheme(String scheme, Uri uri) {
    // 如果 scheme 是 null，就给它一个默认的 scheme
    return Uri.parse(fixUrlScheme(scheme, uri.toString()));
  }

  /// 给 url 添加一个 scheme ———— 如果它没有的话
  static String fixUrlScheme(String scheme, String originalUrl) {
    if (originalUrl.indexOf("://") > 0) {
      return originalUrl;
    }
    StringBuffer sb = new StringBuffer();
    if (originalUrl.startsWith("://")) {
      sb..write(scheme)..write(originalUrl);
    } else if (originalUrl.startsWith("//")) {
      sb..write(scheme)..write(":")..write(originalUrl);
    } else if (originalUrl.startsWith("/")) {
      sb..write(scheme)..write(":/")..write(originalUrl);
    } else {
      sb..write(scheme)..write("://")..write(originalUrl);
    }
    return sb.toString();
  }

  /// 将 Map 中的值填充到参数化的 url 中去。
  ///
  /// 参数化的 url 是这样的： zhihu://answer/{answerId}/reviews?{num:int}&{time?:long}
  static String replaceParamInUrl(Map map, String str) {
    StringBuffer builder = new StringBuffer();
    int index = 0;
    while (index != -1 && index < str.length) {
      //左大括号 { 的位置
      int indexLeft = str.indexOf("{", index);
      if (indexLeft == -1) {
        break;
      }
      //右大括号 } 的位置
      int indexRight = str.indexOf("}", indexLeft);
      if (indexRight == -1) {
        // 说明 query 中可能只包含了一个 {，这是合法的
        break;
      }
      // 找到 key,key 的右侧是 ?:=} 之一
      String key = findKey(str.substring(indexLeft + 1, indexRight));
      //必处不可能为 null，此处由编译期保证
      String value = map[key];
      builder.write(str.substring(index, indexLeft));
      String leftChar = str[indexLeft - 1];
      if (leftChar == '/' || leftChar == '=') {
        // leftChar 为 / 说明是 segment 而 leftChar 为 = 说明是 queryKey={key?:type=value} 形式的 query
        builder.write(value);
      } else if (leftChar == '?' || leftChar == '&') {
        // leftChar 为 ? 或者 & 说明它是一个普通的 query
        builder..write(key)..write("=")..write(value);
      }
      index = indexRight + 1;
    }
    builder.write(str.substring(index));
    return builder.toString();
  }

  /// 在 {key?:type=value} 中寻找 key （不包含大括号）
  ///
  /// 只查找 ?:=，这三个字符一个比一个靠前，所以只要按顺序找到一个就算找到 key 的右侧边界了
  static String findKey(String subString) {
    // crtIndex 表示 key 的右侧边界
    int crtIndex = subString.length;
    int questionMarkIndex = subString.indexOf("?");
    if (questionMarkIndex > 0) {
      crtIndex = questionMarkIndex;
    } else {
      int colonIndex = subString.indexOf(":");
      if (colonIndex > 0) {
        crtIndex = colonIndex;
      } else {
        int equalIndex = subString.indexOf("=");
        if (equalIndex > 0) {
          crtIndex = equalIndex;
        }
      }
    }
    return subString.substring(0, crtIndex);
  }
}