import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/constant/value.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';
import 'package:zrouter/src/compiler/compiler_data.dart';

import '../../annotations.dart';
import 'route_element.dart';
import 'extensions.dart';
import 'compiler_util.dart';

/// 用于处理单个 PageRoute 注册，处理后存储到某个变量里面，供 RouteMapperGenerator 读取
class RouteReader extends Generator {

  @override
  String generate(LibraryReader library, BuildStep buildStep) {
    var elements = getAnnotationElements(library, TypeChecker.fromRuntime(ScreenRoute));
    if (elements.isNotEmpty) {
      elements.forEach((it) => process(it, library, buildStep));
    }
    return null;
  }

  void process(AnnotatedElement ae, LibraryReader lib, BuildStep buildStep) {
    var element = ae.element;
    if (element is ClassElement || element is FunctionElement) {
      CompilerData.allRoutes
          .add(RouteElement(lib, ae.annotation, ae.element, buildStep));
    } else {
      throw UnsupportedError(
          "ZRouter 注解只能在类和顶级方法上使用，当前是： ${element.runtimeType} ${locateElement(element, buildStep)}");
    }
  }

  /// 获取一个 library 上的所有同一类型的注解，如果某个类上有多个注解，同样如此
  Iterable<AnnotatedElement> getAnnotationElements(LibraryReader lib,
      TypeChecker checker,
      {bool throwOnUnresolved}) sync* {
    // TODO 优化，这里不使用 lib.allElements 而是只取 ClassElement
    for (final element in lib.allElements) {
      final annotations = allAnnotationsOf(element, checker,
          throwOnUnresolved: throwOnUnresolved);
      if (annotations != null) {
        for (var value in annotations.toList()) {
          yield AnnotatedElement(ConstantReader(value), element);
        }
      }
    }
  }

  /// 获取一个 element 上的所有同一类型的注解
  Iterable<DartObject> allAnnotationsOf(Element element, TypeChecker checker,
      {bool throwOnUnresolved}) {
    if (element.metadata.isEmpty) {
      return null;
    }
    return checker.annotationsOf(element, throwOnUnresolved: throwOnUnresolved);
  }
}
